﻿# -*- coding: UTF-8 -*-
import csv, uuid, os, zipfile, string, platform


label_key = 'KEY'


def lista_colunas(arquivo_original, delimitador):
	'''
	Abre arquivo csv original e adiciona uma chave única a cada registro. Retorna lista de registros (cada registro é um dicionário) e lista com elementos do cabeçalho do arquivo original.
	'''
	try:
		with open(arquivo_original) as arquivo:
			dict_reader = csv.DictReader(arquivo, delimiter=str(delimitador))
			# dict_reader = csv.DictReader(arquivo, delimiter=str(delimitador), dialect='excel')
			headers_arquivo = dict_reader.fieldnames
			linhas = []
			for l in dict_reader:
				if None not in l.keys() and None not in l.values():
					key = uuid.uuid4()
					l[label_key] = str(key)
					linhas.append(l)
		arquivo.close()
		# print(len(linhas))
		dct = {'linhas': linhas, 'headers': headers_arquivo}
		return dct
	except (IOError, TypeError, csv.Error) as e:
		print ('Erro ao manipular arquivo original: %s' % e)
	except Exception as exception:
		print ('Verifique o arquivo csv de entrada: %s' % exception)

def prepara_dados(dados, campos_selecionados):
	'''
	Recebe lista dos dados do arquivo original e as colunas a serem geocodificadas. Retorna lista de dicionários com a chave e com as colunas recebidas.
	'''
	try:
		dados_pesquisa = []
		for d in dados:
			dct = {}
			dct[label_key] = d[label_key]
			for c in campos_selecionados:
				if c in list(d.keys()):
					dct[c] = d[c]
			dados_pesquisa.append(dct)
		return dados_pesquisa
	except Exception as e:
		print ('Erro ao preparar dados para geocodificação: %s' % e)

def padroniza_dados(dados):
	'''
	Recebe lista de dicionários com chave e colunas a serem geocodificadas, aplica a função 'troca_verbetes' para cada coluna e retorna nova lista.
	'''
	try:
		for d in dados:
			c = d.keys()
			chaves = list(c)
			chaves.remove(label_key)
			for k in chaves:
				valor = d[k]
				d[k] = _troca_verbetes(valor)
		return dados
	except Exception as e:
		print ('Erro ao padronizar dados para geocodificação: %s' % e)


def _troca_verbetes(dado):
	'''
	Recebe uma string e substitui verbetes comuns por sinônimos
	'''
	s = dado.split()
	for x in range(0, len(s)):
		if s[x].lower() in ['conjunto','conj','conj.','cj','cj.']:
			s[x] = 'CJ'
		elif s[x].lower() in ['lote','lt','lt.']:
			s[x] = 'LT'
		elif s[x].lower() in ['loja','lj','lj.']:
			s[x] = 'LJ'
		elif s[x].lower() in ['bloco','bl','bl.']:
			s[x] = 'BL'
		elif s[x].lower() in ['q.','qd','qd.','quadra']:
			s[x] = 'QD'
		elif s[x].lower() in ['apartamento','apto','apt','apt.','ap','ap.']:
			s[x] = 'APT'
		elif s[x].lower() in ['condomínio','condominio','cond','cond.']:
			s[x] = 'Cond'
		elif s[x].lower() in ['rua','r.']:
			s[x] = 'Rua'
		elif s[x].lower() in ['alameda','al','al.']:
			s[x] = 'Alameda'
		elif s[x].lower() in ['setor','set','set.','st','st.']:
			s[x] = 'Setor'
		elif s[x].isdigit():
			num_int = int(s[x])
			s[x] = str(num_int)
	return ' '.join(s)

def gera_arquivo(lista_final, prefixo, dir_arquivo, labels):
	'''
	Recebe lista final e lista de argumentos da execução (nome do arquivo original e colunas a serem manipuladas) e grava novo arquivo .csv
	'''
	try:
		separador = '\\' if platform.system() == 'Windows' else '/'
		dct = _identifica_diretorio(dir_arquivo)
		diretorio_saida = dct['diretorio']
		nome_arquivo = dct['nome_arquivo']
		diretorio = diretorio_saida + 'files_' + nome_arquivo.replace('.csv', '') + separador

		if not os.path.exists(diretorio):
			os.makedirs(diretorio)

		novo_arquivo = diretorio + prefixo + '_' + nome_arquivo

		with open(novo_arquivo, 'w', newline='') as arquivo:

			if label_key not in labels:
				labels.insert(0, label_key)

			dict_writer = csv.DictWriter(arquivo, fieldnames=labels, delimiter=';')
			# dict_writer = csv.DictWriter(arquivo, fieldnames=labels, delimiter=';', dialect='excel')
			dict_writer.writeheader()
			for l in lista_final:
				dict_writer.writerow(l)
		arquivo.close()
		print ('Arquivo ' + prefixo + '_' + nome_arquivo + ' criado.')
	except (IOError, TypeError, csv.Error) as e:
		print ('Erro ao criar arquivo: %s' % e)


def _identifica_diretorio(arquivo):
	try:
		separador = '\\' if platform.system() == 'Windows' else '/'
		dr = arquivo.split(separador)
		arq = dr[-1]
		dr.pop()
		diretorio = separador.join(dr) + separador
		# print(diretorio, arq)
		dct = {'diretorio': diretorio, 'nome_arquivo': arq}
		return dct
	except Exception as e:
		print ('Erro ao manipular o diretório: %s' % e)
